from flask import abort, current_app, flash, redirect, render_template, url_for
from flask_login import current_user, login_required
from ..models import Corpus, User, Job
from ..tables import AdminUserTable, AdminUserItem
from . import main
from .forms import CreateCorpusForm
from ..decorators import admin_required
from .. import db
import os


@main.route('/')
def index():
    return render_template('main/index.html.j2', title='Opaque')


@main.route('/admin', methods=['GET', 'POST'])
@login_required
@admin_required
def for_admins_only():
    users = User.query.order_by(User.username).all()
    items = [AdminUserItem(u.username, u.email, u.role_id, u.confirmed) for u in users]
    table = AdminUserTable(items).__html__()  # converts table object to html string
    table = table.replace('tbody', 'tbody class="list"', 1)  # add class list to tbody element. Needed by list.js
    return render_template('main/admin.html.j2', title='Administration tools',
                           table=table)


@main.route('/dashboard', methods=['GET', 'POST'])
@login_required
def dashboard():
    create_corpus_form = CreateCorpusForm()

    if create_corpus_form.validate_on_submit():
        app = current_app._get_current_object()
        corpus = Corpus(creator=current_user._get_current_object(),
                        description=create_corpus_form.description.data,
                        title=create_corpus_form.title.data)
        db.session.add(corpus)
        db.session.commit()

        dir = os.path.join(app.config['OPAQUE_STORAGE'],
                           str(corpus.user_id),
                           'corpora',
                           str(corpus.id))

        try:
            os.makedirs(dir)
        except OSError:
            flash('OSError!')
        else:
            for file in create_corpus_form.files.data:
                file.save(os.path.join(dir, file.filename))
            flash('Corpus created!')
        return redirect(url_for('main.dashboard'))

    return render_template(
        'main/dashboard.html.j2',
        title='Dashboard',
        create_corpus_form=create_corpus_form
    )


@main.route('/jobs/<int:job_id>')
@login_required
def job(job_id):
    job = Job.query.filter_by(id=job_id).first()
    if not job:
        print('Job not found.')
        abort(404)
    elif not job.user_id == current_user.id:
        print('Job does not belong to current user.')
        abort(403)
    input_dir = os.path.join(current_app.config['OPAQUE_STORAGE'],
                             str(current_user.id),
                             'jobs',
                             str(job.id))
    output_dir = os.path.join(current_app.config['OPAQUE_STORAGE'],
                             str(current_user.id),
                             'jobs',
                             str(job.id),
                             'output')

    input_files = {}
    for file in os.listdir(input_dir):
        if file == 'output':
            continue
        input_files[file] = os.path.join(input_dir, file)

    output_files = {}
    if job.status == 'complete':
        for input_file in input_files:
            dir = os.path.join(output_dir, input_file)
            for output_file in os.listdir(dir):
                output_files[output_file] = os.path.join(dir, output_file)

    return render_template('main/jobs/job.html.j2',
                           input_files=input_files,
                           job=job,
                           output_files=output_files, title='Job')
